#
# XMLWriter.py
#
# Generate arbitrary XML files
#
# (c) 2004, Davyd Madeley <davyd@ucc.asn.au>
#

import time

class PlanetItem:
	def __init__(self, blog, item):
		self.itemTitle	= item.itemTitle
		self.itemURL	= item.itemURL
		self.itemDate	= item.itemDate
		self.blogTitle	= blog.blogTitle
		self.blogURL	= blog.blogURL
		self.imageURL	= blog.imageURL
		self.imageLink	= blog.imageLink
		self.contents	= item.contents

class PlanetDate:
	def __init__(self, date):
		self.planetDate	= date
		self.items	= []

class Planet:
	def __init__(self, bloglist):
		self.__bloglist__	= bloglist
		self.__tainted__	= True
		self.dates		= []
		
	def append(self, blog):
		self.__bloglist__.append(blog)
		self.__tainted__	= True
	
	def __getNext__(self, bloglist):
		"Returns a PlanetItem reaped from a bloglist"
		latestTime		= 0
		holdingBlog		= None
		for blog in bloglist:
			if len(blog.items) > 0 and blog.items[0].itemDate > latestTime:
				latestTime	= blog.items[0].itemDate
				holdingBlog	= blog
		if holdingBlog == None:
			return None
		item	= holdingBlog.items.pop(0)
		return PlanetItem(holdingBlog, item)
	
	def sort(self):
		if self.__tainted__:
			# we need to sort the blogs into a single news feed
			# copy the bloglist to a working symbol
			bloglist	= self.__bloglist__ + []
			lastDate	= -1
			workingDate	= None
			self.dates	= []
			while True:
				lastItem	= self.__getNext__(bloglist)
				if lastItem == None:
					break
				# this checks to see if it's a new day
				if time.localtime(lastItem.itemDate)[6] != lastDate:
					lastDate	= time.localtime(lastItem.itemDate)[6]
					workingDate	= PlanetDate(lastItem.itemDate)
					self.dates.append(workingDate)
				# append the item to the current date
				workingDate.items.append(lastItem)	
			self.__tainted__	= False
		return self.dates
			

class XMLWriter:
	def __init__(self, doctype, bloglist):
		self.blogs	= bloglist
		self.planet	= Planet(bloglist)
		self.items	= self.planet.sort()
		# doctype should be something like XMLWriter.XHTMLWriter
		self.writer		= doctype(self.items)
		self.writer.parent	= self
	
	def write(self):
		output		= self.writer.write()
		return output

class XHTMLWriter:
	def __init__(self, planet):
		self.planet	= planet
		self.maxitems	= 50
		self.parent	= None
	
	def __write_item__(self, item):
		output	=	''
		output	+=	'<div class="item">\n'
		output	+=	'<h2>%s: %s</h2>\n' % (item.blogTitle, item.itemTitle)
		output	+=	'<p class="time">\n'
		if item.imageURL:
			output += '<a class="image" href="%s"><img class="image" src="%s" alt="%s" /></a>\n' % (item.imageLink, item.imageURL, item.blogTitle)
		output	+=	'(<a href="%s">%s</a>)\n' % (item.itemURL ,time.strftime('%A %B %d, %Y %H:%M AWST', time.localtime(item.itemDate)))
		output	+=	'</p>\n'
		output	+=	'<p class="body">\n'
		output	+=	item.contents
		output	+=	'\n</p>\n'
		output	+=	'</div>\n'
		return output
	
	def write(self):
		itemcount	= 0
		output	=	''
		output	+=	'<?xml version="1.0" encoding="UTF-8"?>\n'
		output	+=	'<!DOCTYPE html PUBLIC "-//W3C//DTD XHTML 1.1//EN" "http://www.w3.org/TR/xhtml11/DTD/xhtml11.dtd">\n'
		output	+=	'<html xmlns="http://www.w3.org/1999/xhtml" xml:lang="en" >\n'
		output	+=	'<head>\n'
		output	+=	'<title>Planet UCC</title>\n'
		output	+=	'<link rel="stylesheet" href="planet.css" type="text/css" media="screen" />\n'
		output	+=	'<link rel="icon" type="image/png" href="icon.png" />\n'
		output	+=	'</head>\n'
		output	+=	'<body>\n'
		output	+=	'<div class="header">\n'
		try:
			output	+=	open('header.html').read()
		except:
			pass
		output	+=	'</div>\n'
		output	+=	'<div class="sidebar">\n'
		try:
			output	+=	open('sidebar.html').read()
		except:
			pass
		output	+=	'<h2>Last Updated</h2>\n'
		output	+=	'<p>\n'
		output	+=	'%s\n' % time.strftime('%A %B %d, %Y %H:%M AWST', time.localtime())
		output	+=	'</p>\n'
		if self.parent:
			output	+=	'<h2>Feeds</h2>\n'
			output	+=	'<p>\n'
			blogscopy	= self.parent.blogs + []
			blogscopy.sort(self.__blog_sort__)
			for blog in blogscopy:
				output	+= '<a href="%s">%s</a> (<a href="%s">feed</a>)<br />\n' % (blog.blogURL, blog.blogTitle, blog.feedURL)
			output	+=	'</p>\n'
		output	+=	'</div>\n'
		output	+=	'<div class="items">\n'
		for date in self.planet:
			output	+= '<h1>%s</h1>\n' % time.strftime('%A %B %d, %Y', time.localtime(date.planetDate))
			for item in date.items:
				output	+= self.__write_item__(item)
				# see how many items we've written
				itemcount += 1
				if itemcount >= self.maxitems:
					break
			# again, check to see if we've written the maximum number of items
			if itemcount >= self.maxitems:
				break
		output	+=	'</div>\n'
		output	+=	'<div class="footer">\n'
		try:
			output	+= 	open('footer.html').read()
		except:
			pass
		output	+=	'</div>\n'
		output	+=	'</body>\n'
		output	+=	'</html>'
		return output
	
	def __blog_sort__(self, blog1, blog2):
		name1, name2	= blog1.blogTitle.split(' ')[-1], blog2.blogTitle.split(' ')[-1]
		if name1 < name2: return -1
		if name1 == name2: return 0
		if name1 > name2: return 1
